<?php

/**
	* SmartPreview main controller
	*
	* @package boardtalk\smartpreview
	* @copyright (c) 2024 Colette Byrne
	* @license GNU General Public License, version 2 (GPL-2.0)
	*/

namespace boardtalk\smartpreview\controller;

use phpbb\template\template;
use phpbb\controller\helper;
use phpbb\db\driver\driver_interface;
use phpbb\user;
use phpbb\request\request_interface;
use phpbb\auth\auth;
use phpbb\user_loader;
use phpbb\path_helper;
use boardtalk\smartpreview\service\topic_provider;
use Symfony\Component\HttpFoundation\Response;

/**
	* Controller handling the recent topics feed
	*/

class main_controller
{
	protected $template;
	protected $helper;
	protected $db;
	protected $user;
	protected $request;
	protected $auth;
	protected $topic_provider;
	protected $user_loader;
	protected $php_ext;

	/**
	* Constructor
	*
	* @param template		   $template Template object
	* @param helper		   $helper	 Controller helper
	* @param driver_interface $db	   Database connection
	* @param user			 $user	   User object
	* @param request_interface $request Request object
	* @param auth			 $auth	   Auth object
	* @param path_helper			 $path_helper Path helper
	*/
	public function __construct(template $template, helper $helper, driver_interface $db, user $user, request_interface $request, auth $auth, topic_provider $topic_provider, user_loader $user_loader, path_helper $path_helper)
	{
		$this->template = $template;
		$this->helper = $helper;
		$this->db = $db;
		$this->user = $user;
		$this->request = $request;
		$this->auth = $auth;
		$this->topic_provider = $topic_provider;
		$this->user_loader = $user_loader;
		$this->php_ext = $path_helper->get_php_ext();
	}

	/**
	* Display the recent topics feed
	*
	* @return Response
	*/
	public function handle()
	{
		$this->user->add_lang_ext('boardtalk/smartpreview', 'common');
		$user_id = (int) $this->user->data['user_id'];
		// 1) Fetch latest topics with pagination
		$limit = 15;
		$start = max(0, (int) $this->request->variable('start', 0));
		$result = $this->topic_provider->fetch_recent_topics($start, $limit);
		$topics = $result['topics'];
		$post_ids = $result['post_ids'];
		$more = $result['more'];

		// 3) Fetch first & last posts + avatars + parser flags
		if (!empty($post_ids))
		{
			$ids_sql = $this->db->sql_in_set('p.post_id', array_unique($post_ids));
			$sql = 'SELECT p.post_id, p.poster_id, p.post_text, p.post_time,
												   p.bbcode_uid, p.bbcode_bitfield,
												   p.enable_bbcode, p.enable_smilies, p.enable_magic_url,
												   u.username, u.user_colour
										FROM ' . POSTS_TABLE . ' p
										LEFT JOIN ' . USERS_TABLE . ' u
										  ON p.poster_id = u.user_id
										WHERE ' . $ids_sql;
			$postsResult = $this->db->sql_query($sql);

			while ($row = $this->db->sql_fetchrow($postsResult))
			{
				$pid = (int) $row['post_id'];
				foreach ($topics as $tid => &$topic)
				{
					if ($pid === $topic['first_id'])
					{
						$topic['first_poster_id'] = (int) $row['poster_id'];
						$topic['first_user'] = get_username_string(
							'full',
							(int) $row['poster_id'],
							$row['username'],
							$row['user_colour']
						);
						$topic['first_username'] = $row['username'];
						$topic['first_text'] = $row['post_text'];
						$topic['first_time'] = (int) $row['post_time'];
						$topic['first_bbcode_uid'] = $row['bbcode_uid'];
						$topic['first_bbcode_bitfield'] = $row['bbcode_bitfield'];
						$topic['first_enable_bbcode'] = (int) $row['enable_bbcode'];
						$topic['first_enable_smilies'] = (int) $row['enable_smilies'];
						$topic['first_enable_magic_url'] = (int) $row['enable_magic_url'];
						$topic['first_avatar'] = $this->user_loader->get_avatar((int) $row['poster_id'], true);
					}
					if ($pid === $topic['last_id'])
					{
						$topic['last_poster_id'] = (int) $row['poster_id'];
						$topic['last_user'] = get_username_string(
							'full',
							(int) $row['poster_id'],
							$row['username'],
							$row['user_colour']
						);
						$topic['last_text'] = $row['post_text'];
						$topic['last_time'] = (int) $row['post_time'];
						$topic['last_bbcode_uid'] = $row['bbcode_uid'];
						$topic['last_bbcode_bitfield'] = $row['bbcode_bitfield'];
						$topic['last_enable_bbcode'] = (int) $row['enable_bbcode'];
						$topic['last_enable_smilies'] = (int) $row['enable_smilies'];
						$topic['last_enable_magic_url'] = (int) $row['enable_magic_url'];
					}
				}
				unset($topic);
			}
			$this->db->sql_freeresult($postsResult);
		}

		// 4) Determine which reply to show (first unread or last)
		//	  and compute unread state for each topic
		foreach ($topics as $tid => &$topic)
		{
			$topic['reply_exists'] = false;
			$topic['reply_label'] = '';
			$topic['reply_id'] = 0;
			$topic['reply_user'] = '';
			$topic['reply_time'] = 0;
			$topic['reply_text'] = '';
			$topic['reply_bbcode_uid'] = '';
			$topic['reply_bbcode_bitfield'] = '';
			$topic['reply_enable_bbcode'] = 0;
			$topic['reply_enable_smilies'] = 0;
			$topic['reply_enable_magic_url'] = 0;

					   // mark time provided by topic_provider via phpBB tracking
					   $mark_time = (int) $topic['mark_time'];

					   $topic['is_unread'] = ($topic['last_post_time'] > $mark_time);
					   $topic['first_is_unread'] = ($topic['first_time'] > $mark_time);

					   if ($topic['replies'] > 0)
					   {
							   $found_unread = false;

							   if ($topic['last_post_time'] > $mark_time)
							   {

					// fetch first post _after_ mark_time
					$sql = 'SELECT p.post_id, p.poster_id, p.post_text, p.post_time,
								   p.bbcode_uid, p.bbcode_bitfield,
								   p.enable_bbcode, p.enable_smilies, p.enable_magic_url,
								   u.username, u.user_colour
							FROM ' . POSTS_TABLE . " p
							LEFT JOIN " . USERS_TABLE . " u
							  ON p.poster_id = u.user_id
							WHERE p.topic_id = $tid
							  AND p.post_time > $mark_time
							  AND p.post_id <> {$topic['first_id']}
							ORDER BY p.post_time ASC";
					$unreadPostResult = $this->db->sql_query_limit($sql, 1);
					if ($row2 = $this->db->sql_fetchrow($unreadPostResult))
					{
						$found_unread = true;
						$topic['reply_id'] = (int) $row2['post_id'];
						$topic['reply_user'] = get_username_string(
							'full',
							(int) $row2['poster_id'],
							$row2['username'],
							$row2['user_colour']
						);
						$topic['reply_time'] = (int) $row2['post_time'];
						$topic['reply_text'] = $row2['post_text'];
						$topic['reply_bbcode_uid'] = $row2['bbcode_uid'];
						$topic['reply_bbcode_bitfield'] = $row2['bbcode_bitfield'];
						$topic['reply_enable_bbcode'] = (int) $row2['enable_bbcode'];
						$topic['reply_enable_smilies'] = (int) $row2['enable_smilies'];
						$topic['reply_enable_magic_url'] = (int) $row2['enable_magic_url'];
						$topic['reply_label'] = $this->user->lang('SMARTPREVIEW_FIRST_UNREAD');
					}
					$this->db->sql_freeresult($unreadPostResult);
				}

				if (!$found_unread)
				{
					// fallback to last reply
					$topic['reply_id'] = $topic['last_id'];
					$topic['reply_user'] = $topic['last_user'];
					$topic['reply_time'] = $topic['last_time'];
					$topic['reply_text'] = $topic['last_text'];
					$topic['reply_bbcode_uid'] = $topic['last_bbcode_uid'];
					$topic['reply_bbcode_bitfield'] = $topic['last_bbcode_bitfield'];
					$topic['reply_enable_bbcode'] = $topic['last_enable_bbcode'];
					$topic['reply_enable_smilies'] = $topic['last_enable_smilies'];
					$topic['reply_enable_magic_url'] = $topic['last_enable_magic_url'];
					$topic['reply_label'] = '';
				}

				$topic['reply_exists'] = true;
			}
		}
		unset($topic);

		// 5) Parse, truncate (30 words), and assign everything to template
		foreach ($topics as $tid => $topic)
		{
			// ── FIRST post ──
			$opts1 = 0;
			if ($topic['first_enable_bbcode'])
			{
				$opts1 |= OPTION_FLAG_BBCODE;
			}
			if ($topic['first_enable_smilies'])
			{
				$opts1 |= OPTION_FLAG_SMILIES;
			}
			if ($topic['first_enable_magic_url'])
			{
				$opts1 |= OPTION_FLAG_LINKS;
			}

			$parsed1 = generate_text_for_display(
				$topic['first_text'],
				$topic['first_bbcode_uid'],
				$topic['first_bbcode_bitfield'],
				$opts1
			);
			$words1 = preg_split('/\s+/', strip_tags($parsed1), -1, PREG_SPLIT_NO_EMPTY);
			$first_trunc = count($words1) > 30;
			$first_excerpt = $first_trunc
					? implode(' ', array_slice($words1, 0, 30)) . '...'
					: $parsed1;

			// ── REPLY ──
			$reply_excerpt = '';
			$reply_trunc = false;
			if ($topic['reply_exists'])
			{
				$opts2 = 0;
				if ($topic['reply_enable_bbcode'])
				{
					$opts2 |= OPTION_FLAG_BBCODE;
				}
				if ($topic['reply_enable_smilies'])
				{
					$opts2 |= OPTION_FLAG_SMILIES;
				}
				if ($topic['reply_enable_magic_url'])
				{
					$opts2 |= OPTION_FLAG_LINKS;
				}

				$parsed2 = generate_text_for_display(
					$topic['reply_text'],
					$topic['reply_bbcode_uid'],
					$topic['reply_bbcode_bitfield'],
					$opts2
				);
				$words2 = preg_split('/\s+/', strip_tags($parsed2), -1, PREG_SPLIT_NO_EMPTY);
				$reply_trunc = count($words2) > 30;
				$reply_excerpt = $reply_trunc
						? implode(' ', array_slice($words2, 0, 30)) . '...'
						: $parsed2;
			}

			// new‐replies count
			$new_replies = isset($topic['new_replies']) ? $topic['new_replies'] : 0;

			$this->template->assign_block_vars('topics', [
				// avatar
				'HAS_AVATAR' => (bool) $topic['first_avatar'],
				'AVATAR' => $topic['first_avatar'],

				// first‐post info
				'TOPIC_TITLE' => $topic['title'],
				'U_FIRST_POST' => append_sid('viewtopic.' . $this->php_ext, 't=' . $tid),
				'FIRST_POSTER_FULL' => $topic['first_user'],
				'FIRST_POSTER' => $topic['first_username'],
				'FIRST_POST_TIME' => $this->user->format_date($topic['first_time']),
				'FIRST_POST_EXCERPT' => $first_excerpt,
				'FIRST_TRUNCATED' => (int) $first_trunc,

				// stats
				'TOPIC_VIEWS' => $topic['views'],
				'TOPIC_REPLIES' => $topic['replies'],
				'TOPIC_NEW_REPLIES' => $topic['new_replies'],
				'IS_UNREAD' => $topic['is_unread'],
				'FIRST_IS_UNREAD' => $topic['first_is_unread'],

				// reply info
				'HAS_REPLY' => (int) $topic['reply_exists'],
				'REPLY_LABEL' => $topic['reply_label'],
				'U_REPLY_POST' => append_sid('viewtopic.' . $this->php_ext, 'p=' . $topic['reply_id']) . '#p' . $topic['reply_id'],
				'REPLY_POSTER' => $topic['reply_user'],
				'REPLY_TIME' => $this->user->format_date($topic['reply_time']),
				'REPLY_EXCERPT' => $reply_excerpt,
				'REPLY_TRUNCATED' => (int) $reply_trunc,
			]);
		}

		$this->template->assign_vars([
			'U_LOAD_MORE' => $this->helper->route('smartpreview_recent', ['start' => $start + $limit]),
			'SHOW_LOAD_MORE' => $more,
			'U_SMARTPREVIEW' => $this->helper->route('smartpreview_recent'),
		]);

		return $this->helper->render(
			'recent_body.html',
			$this->user->lang('SMARTPREVIEW_TITLE')
		);
	}
}
