<?php

/**
	* Topic provider service
	*
	* @package boardtalk\smartpreview
	* @copyright (c) 2024 Colette Byrne
	* @license GNU General Public License, version 2 (GPL-2.0)
	*/

namespace boardtalk\smartpreview\service;

use phpbb\db\driver\driver_interface;
use phpbb\user;
use phpbb\auth\auth;

class topic_provider
{
	/** @var driver_interface */
	protected $db;

	/** @var user */
	protected $user;

	/** @var auth */
	protected $auth;

	public function __construct(driver_interface $db, user $user, auth $auth)
	{
		$this->db = $db;
		$this->user = $user;
		$this->auth = $auth;
	}

	/**
	 * Fetch recent topics with reply counters
	 *
	 * @param int $start Pagination start
	 * @param int $limit Number of topics to retrieve
	 * @return array [topics, post_ids, more]
	 */
	public function fetch_recent_topics(int $start, int $limit): array
	{
		$user_id = (int) $this->user->data['user_id'];

		$readable_forums = array_keys($this->auth->acl_getf('f_read', true));
		$forum_sql = $this->db->sql_in_set('t.forum_id', $readable_forums);

			   $sql = 'SELECT t.topic_id, t.forum_id, t.topic_title,
										   t.topic_first_post_id, t.topic_last_post_id,
										   t.topic_last_post_time, t.topic_views
				FROM ' . TOPICS_TABLE . ' t
				WHERE t.topic_visibility = ' . ITEM_APPROVED . '
				  AND ' . $forum_sql . '
				ORDER BY t.topic_last_post_time DESC';

		$res = $this->db->sql_query_limit($sql, $limit + 1, $start);

		$topics = [];
		$post_ids = [];
		$count = 0;
		$more = false;

		while ($row = $this->db->sql_fetchrow($res))
		{
			if ($count === $limit)
			{
				$more = true;
				break;
			}
			$count++;
			$tid = (int) $row['topic_id'];
					   $topics[$tid] = [
							   'title' => $row['topic_title'],
							   'forum_id' => (int) $row['forum_id'],
							   'first_id' => (int) $row['topic_first_post_id'],
							   'last_id' => (int) $row['topic_last_post_id'],
							   'last_post_time' => (int) $row['topic_last_post_time'],
							   'views' => (int) $row['topic_views'],
							   'replies' => 0,
							   'mark_time' => 0,

				// placeholders
				'first_user' => '',
				'first_username' => '',
				'first_text' => '',
				'first_time' => 0,
				'first_avatar' => '',
				'first_poster_id' => 0,
				'first_bbcode_uid' => '',
				'first_bbcode_bitfield' => '',
				'first_enable_bbcode' => 0,
				'first_enable_smilies' => 0,
				'first_enable_magic_url' => 0,
				'last_user' => '',
				'last_text' => '',
				'last_time' => 0,
				'last_poster_id' => 0,
				'last_bbcode_uid' => '',
				'last_bbcode_bitfield' => '',
				'last_enable_bbcode' => 0,
				'last_enable_smilies' => 0,
				'last_enable_magic_url' => 0,
			];

			$post_ids[] = $row['topic_first_post_id'];
			if ($row['topic_last_post_id'] !== $row['topic_first_post_id'])
			{
				$post_ids[] = $row['topic_last_post_id'];
			}
		}
		$this->db->sql_freeresult($res);

		if (!empty($topics))
		{
			$topic_ids = array_keys($topics);
			$sql = 'SELECT topic_id, COUNT(post_id) AS total_posts
					FROM ' . POSTS_TABLE . '
					WHERE ' . $this->db->sql_in_set('topic_id', $topic_ids) . '
					GROUP BY topic_id';
			$reply_res = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($reply_res))
			{
				$tid = (int) $row['topic_id'];
				$topics[$tid]['replies'] = max(0, (int) $row['total_posts'] - 1);
			}
			$this->db->sql_freeresult($reply_res);
		}

			   if (!empty($topics))
			   {
					   $forum_map = [];
					   foreach ($topics as $tid => $topic)
					   {
							   $forum_map[$topic['forum_id']][] = $tid;
					   }

					   $tracking = [];
					   foreach ($forum_map as $fid => $tids)
					   {
							   $tracking += get_complete_topic_tracking($fid, $tids);
					   }

					   foreach ($topics as $tid => &$topic)
					   {
							   $topic['mark_time'] = $tracking[$tid] ?? (int) $this->user->data['user_lastvisit'];

							   if ($topic['replies'] > 0 && $topic['last_post_time'] > $topic['mark_time'])
							   {
									   $sql = 'SELECT COUNT(post_id) AS new_posts
													   FROM ' . POSTS_TABLE . '
													   WHERE topic_id = ' . $tid . '
														 AND post_time > ' . $topic['mark_time'] . '
														 AND post_id <> ' . $topic['first_id'];
									   $new_res = $this->db->sql_query($sql);
									   $topic['new_replies'] = (int) $this->db->sql_fetchfield('new_posts');
									   $this->db->sql_freeresult($new_res);
							   }
							   else
							   {
									   $topic['new_replies'] = 0;
							   }
					   }
					   unset($topic);
			   }

		return [
			'topics' => $topics,
			'post_ids' => $post_ids,
			'more' => $more,
		];
	}
}
